#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e14:cl23[:us]|e15:cl23[:us]> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_clhep_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    clhep_dir=`cd ${CLHEP_BASE_DIR} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

have_qt_build()
{
    # At this time, QT is only available for slf6 and slf7
    osname=`${SSIBUILDSHIMS_DIR}/bin/get_os_platform osname`
    case ${osname} in
      slf6-x86_64)
	have_qt=true
	;;
      slf7-x86_64)
	have_qt=true
	;;
      sl7-x86_64)
	have_qt=true
	;;
      *)
	 echo "ERROR: the QT build of geant4 is only supported for slf6 and slf7"
	 echo "ERROR: attempting to build for ${osname}"
	 exit 1
    esac    
}

make_distribution()
{
    if [ "${maketar}" = "tar" ] 
    then
       if [ -d ${pkgdir}/lib64 ] || [ -d ${pkgdir}/lib ]
       then
	   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
	   exit 0
       fi
    fi
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=geant4
origpkgver=v4_10_4
pkgver=${origpkgver}
ssibuildshims_version=v1_03_01
tarpatchver=`echo ${origpkgver} | cut -f 4 -d"_"`
tarminorver=`echo ${origpkgver} | cut -f 3 -d"_"`
tarmajorver=`echo ${origpkgver} | cut -f 2 -d"_"`
tarbver=`echo ${origpkgver} | cut -f 1-2 -d"_"`
if [[ ${tarpatchver} ]]; then
  tarpatchnum=`echo ${tarpatchver} | sed -e 's/^p0//' | sed -e 's/^b01/0/'`
  tarver=${tarbver}_0${tarminorver}_${tarpatchver}
  g4subdir=Geant4-${tarmajorver}.${tarminorver}.${tarpatchnum}
else
  tarver=${tarbver}_0${tarminorver}
  g4subdir=Geant4-${tarmajorver}.${tarminorver}.0
fi
pkgdotver=`echo ${tarver} | sed -e 's/_/./g' | sed -e 's/^v//'`
#pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=geant${pkgdotver}.tar.gz
patchfile=fermi_geant`echo ${origpkgver} | sed -e 's/^v//'`.patch
#pkgbasedir=`echo ${origpkgver} | sed -e 's/^v4_//'| sed -e 's/_/./g' | sed -e 's/00/0/' | sed -e 's/p0//'`
echo
echo "tarver: ${tarver}"
echo "g4subdir: ${g4subdir}"
echo

get_this_dir

get_ssibuildshims

source define_basics --

make_distribution

echo "building ${package} for ${OS}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [[ "${basequal}" == e1[0245]:cl23 ]]
then
  cxxflg="c++14"
elif [[ "${basequal}" == e1[0245]:cl23:qt  ]]
then
  cxxflg="c++14"
  have_qt_build
elif [[ "${basequal}" == e1[0245]:cl23:us  ]]
then
  cxxflg="c++14"
  use_usolids="-DGEANT4_USE_USOLIDS=ON"
elif [[ "${basequal}" == e1[0245]:cl23:qt:us  ]]
then
  cxxflg="c++14"
  have_qt_build
  use_usolids="-DGEANT4_USE_USOLIDS=ON"
else
  echo "ERROR: unrecognized basequal ${basequal}"
  usage
  exit 1
fi

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
# make sure we start from an empty build directory
if [ -d ${blddir} ]; then rm -rf ${blddir}; fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# pull the geant4 datasets
${thisdir}/scripts/getG4DataSets.sh ${product_dir} || ssi_die "ERROR: failed to install datasets"

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual} || ssi_die "ERROR: fake declare failed"

set -x

cd ${sourcedir}
tar xzf ${tardir}/${pkgtarfile} || ssi_die "ERROR: failed to unwind the source."

# apply Fermi patches
#patch -b -p0 < ${patchdir}/${patchfile} || ssi_die "ERROR: patch failed."
# bug fix patch
##cd ${sourcedir}/geant${pkgdotver}  || ssi_die "cannot cd to ${sourcedir}/geant${pkgdotver}"
##patch -b -p0 < ${patchdir}/geant4_10.3_and_up.patch || ssi_die "ERROR: patch geant4_10.2.p03_and_up.patch failed."

set +x

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed."

# We need cmake
source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake || ssi_die "ERROR: failed to setup camke"

# make sure we have vecgeom for usolids
if [[ ${use_usolids} ]]
then
  if [ -z ${VECGEOM_DIR} ] 
  then
    echo "ERROR: VECGEOM_DIR is undefined, cannot build"
    exit 1
  else
    if [ -z ${USolids_DIR} ]; then
    export USolids_DIR=${VECGEOM_FQ_DIR}/lib/CMake/USolids/
    fi
  fi
fi 

# define use_qt here
if [[ ${have_qt} ]]
then
  if [ -z ${QT_FQ_DIR} ] 
  then
    echo "ERROR: QT_FQ_DIR is undefined, cannot make a QT build"
    exit 1
  fi
  use_qt="-DGEANT4_USE_QT=ON -DQT_QMAKE_EXECUTABLE=${QT_FQ_DIR}/bin/qmake"
fi

export GEANT4_SOURCE=${GEANT4_DIR}/source/geant${pkgdotver}
export BLDDIR=${blddir}
GEANT4_J=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x

cd ${BLDDIR}
env CC=gcc CXX=g++ FC=gfortran cmake -DGEANT4_BUILD_CXXSTD="${cxxflg}" \
                                     -DCMAKE_INSTALL_PREFIX=${GEANT4_FQ_DIR} \
				     -DGEANT4_USE_SYSTEM_CLHEP=ON \
				     -DCLHEP_ROOT_DIR:STRING="${CLHEP_BASE_DIR}" \
				     -DBUILD_STATIC_LIBS=ON \
				     -DGEANT4_USE_OPENGL_X11=ON \
				     -DGEANT4_USE_GDML=ON \
				     ${use_usolids} \
				     ${use_qt} \
				     -DXERCESC_ROOT_DIR:STRING="${XERCESCROOT}" \
				     -DCMAKE_BUILD_TYPE=${build_type} \
				     ${GEANT4_SOURCE}
(( $? == 0 )) || ssi_die "ERROR: cmake failed."
make ${GEANT4_J:+-j ${GEANT4_J}} || ssi_die "ERROR: make failed."
make install  || ssi_die "ERROR: make install failed."

set +x

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "ERROR: setup failed."

if [ ! -d ${G4LIB} ]
then
   echo "ERROR: failed to create ${G4LIB}"
   echo "       Something is very wrong"
   exit 1
fi
echo "G4LIB is ${G4LIB}"

# remove build directory from prof distribution
if [ "${extraqual}" = "prof" ]
then
  if [ -d ${pkgdir}/build ]
  then
    rm -rf ${pkgdir}/build
  fi
fi

# need to fix paths:
# bin/geant4-config
# lib64/Geant4-xxx/Geant4*.cmake
# share/Geant4-xxx/geant4make/geant4make*sh
get_clhep_dir
fixfile=/tmp/g4fix$$$$.sh
rm -f ${fixfile}

# get list of files to check
shellFile=`ls ${GEANT4_FQ_DIR}/bin/geant4-config ${GEANT4_FQ_DIR}/bin/*sh ${GEANT4_FQ_DIR}/share/${g4subdir}/geant4make/geant4make*sh`
cmakeConfig=`ls ${G4LIB}/${g4subdir}/Geant4*.cmake`

echo "#!/bin/bash" > ${fixfile}
##echo " set -x " >>  ${fixfile}
for file in ${shellFile}
do
  echo "echo \"fixing ${file}\"" >> ${fixfile}
  echo "mv ${file} ${file}.bak" >> ${fixfile}
  echo "cat ${file}.bak | sed 's%${GEANT4_FQ_DIR}%\${GEANT4_FQ_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${CLHEP_BASE_DIR}%\${CLHEP_BASE_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${clhep_dir}%\${CLHEP_BASE_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${XERCESCROOT}%\${XERCESCROOT}%g' \\" >> ${fixfile}
  if [[ ${use_usolids} ]]
  then
  echo "    | sed 's%${VECGEOM_FQ_DIR}%\${VECGEOM_FQ_DIR}%g' \\" >> ${fixfile}         
  fi
  if [[ ${have_qt} ]]
  then
  echo "  | sed 's%${QT_FQ_DIR}%\${QT_FQ_DIR}%g'   \\" >> ${fixfile}    
  fi
  echo "     > ${file} || exit 1" >> $fixfile
  echo "chmod +x  ${file}" >> ${fixfile}
done
for file in ${cmakeConfig}
do
  echo "echo \"fixing ${file}\"" >> ${fixfile}
  echo "mv ${file} ${file}.bak" >> ${fixfile}
  echo "cat ${file}.bak | sed 's%${GEANT4_FQ_DIR}%\$ENV{GEANT4_FQ_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${CLHEP_BASE_DIR}%\$ENV{CLHEP_BASE_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${clhep_dir}%\$ENV{CLHEP_BASE_DIR}%g' \\" >> ${fixfile}
  echo "                | sed 's%${XERCESCROOT}%\$ENV{XERCESCROOT}%g' \\" >> ${fixfile}
  if [[ ${use_usolids} ]]
  then
  echo "    | sed 's%${VECGEOM_FQ_DIR}%\$ENV{VECGEOM_FQ_DIR}%g' \\" >> ${fixfile}         
  fi
  if [[ ${have_qt} ]]
  then
  echo "  | sed 's%${QT_FQ_DIR}%\$ENV{QT_FQ_DIR}%g'   \\" >> ${fixfile}    
  fi
  echo "     > ${file} || exit 1" >> $fixfile
done
chmod +x ${fixfile}
${fixfile} || ssi_die "ERROR: problem running ${fixfile}."

rm -f ${fixfile}

# this must be last
make_distribution

exit 0
